(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory() :
	typeof define === 'function' && define.amd ? define(factory) :
	(factory());
}(this, (function () { 'use strict';

"use strict";
var __extends = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var apollo_cache_1 = require("apollo-cache");
var deepMerge = function (target, source) {
    for (var _i = 0, _a = Object.keys(source); _i < _a.length; _i++) {
        var key = _a[_i];
        if (!!target[key] && typeof target[key] === 'object') {
            deepMerge(target[key], source[key]);
        }
        else {
            target[key] = source[key];
        }
    }
};
var OverrideCache = (function (_super) {
    __extends(OverrideCache, _super);
    function OverrideCache(cache, options) {
        var _this = _super.call(this) || this;
        _this.cache = cache;
        _this.options = options || {};
        return _this;
    }
    OverrideCache.prototype.read = function (query) {
        return this.options.hasOwnProperty('read') ? this.options.read(query) : this.cache.read(query);
    };
    OverrideCache.prototype.write = function (write) {
        this.options.hasOwnProperty('write') ? this.options.write(write) : this.cache.write(write);
    };
    OverrideCache.prototype.diff = function (query) {
        return this.options.hasOwnProperty('diff') ? this.options.diff(query) : this.cache.diff(query);
    };
    OverrideCache.prototype.watch = function (watch) {
        return this.options.hasOwnProperty('watch') ? this.options.watch(watch) : this.cache.watch(watch);
    };
    OverrideCache.prototype.evict = function (query) {
        return this.options.hasOwnProperty('evict') ? this.options.evict(query) : this.cache.evict(query);
    };
    OverrideCache.prototype.reset = function () {
        return this.options.hasOwnProperty('reset') ? this.options.reset() : this.cache.reset();
    };
    OverrideCache.prototype.restore = function (serializedState) {
        return this.options.hasOwnProperty('restore')
            ? this.options.restore(serializedState)
            : this.cache.restore(serializedState);
    };
    OverrideCache.prototype.extract = function (optimistic) {
        return this.options.hasOwnProperty('extract') ? this.options.extract(optimistic) : this.cache.extract(optimistic);
    };
    OverrideCache.prototype.removeOptimistic = function (id) {
        this.options.hasOwnProperty('removeOptimistic')
            ? this.options.removeOptimistic(id)
            : this.cache.removeOptimistic(id);
    };
    OverrideCache.prototype.performTransaction = function (transaction) {
        this.options.hasOwnProperty('performTransaction')
            ? this.options.performTransaction(transaction)
            : this.cache.performTransaction(transaction);
    };
    OverrideCache.prototype.recordOptimisticTransaction = function (transaction, id) {
        this.options.hasOwnProperty('recordOptimisticTransaction')
            ? this.options.recordOptimisticTransaction(transaction, id)
            : this.cache.recordOptimisticTransaction(transaction, id);
    };
    OverrideCache.prototype.transformDocument = function (document) {
        return this.options.hasOwnProperty('transformDocument')
            ? this.options.transformDocument(document)
            : this.cache.transformDocument(document);
    };
    return OverrideCache;
}(apollo_cache_1.ApolloCache));
exports.override = function (cache, options) {
    return new OverrideCache(cache, options);
};
var RoutingCache = (function (_super) {
    __extends(RoutingCache, _super);
    function RoutingCache(caches, router) {
        var _this = _super.call(this) || this;
        _this.caches = caches;
        _this.router = router;
        return _this;
    }
    RoutingCache.prototype.read = function (query) {
        var caches = this.check(this.router(query.query, this.caches));
        if (caches.length === 0) {
            throw new Error('No caches to read from for the document');
        }
        else {
            return caches[0].read(query);
        }
    };
    RoutingCache.prototype.write = function (write) {
        return this.check(this.router(write.query, this.caches)).forEach(function (cache) { return cache.write(write); });
    };
    RoutingCache.prototype.diff = function (query) {
        var caches = this.check(this.router(query.query, this.caches));
        return caches.length === 0 ? { result: null, complete: false } : caches[0].diff(query);
    };
    RoutingCache.prototype.watch = function (watch) {
        var caches = this.check(this.router(watch.query, this.caches));
        return caches.length === 0 ? function () { } : caches[0].watch(watch);
    };
    RoutingCache.prototype.evict = function (query) {
        var result;
        this.check(this.router(query.query, this.caches)).forEach(function (cache) { return (result = cache.evict(query)); });
        return result;
    };
    RoutingCache.prototype.reset = function () {
        return Promise.all(this.caches.map(function (cache) { return cache.reset(); }));
    };
    RoutingCache.prototype.extract = function (optimistic) {
        var state = {};
        for (var idx = 0; idx < this.caches.length; idx++) {
            var cacheKey = '__cache' + idx;
            state[cacheKey] = this.caches[idx].extract(optimistic);
            deepMerge(state, state[cacheKey]);
        }
        return state;
    };
    RoutingCache.prototype.restore = function (serializedState) {
        for (var idx = 0; idx < this.caches.length; idx++) {
            this.caches[idx].restore(serializedState['__cache' + idx]);
        }
        return this;
    };
    RoutingCache.prototype.removeOptimistic = function (id) {
        this.caches.forEach(function (cache) { return cache.removeOptimistic(id); });
    };
    RoutingCache.prototype.performTransaction = function (transaction) {
        var _this = this;
        this.caches.forEach(function (cache) {
            return cache.performTransaction(function () {
                transaction(_this);
            });
        });
    };
    RoutingCache.prototype.recordOptimisticTransaction = function (transaction, id) {
        this.caches.forEach(function (cache) { return cache.recordOptimisticTransaction(transaction, id); });
    };
    RoutingCache.prototype.transformDocument = function (document) {
        return this.caches[0].transformDocument(document);
    };
    RoutingCache.prototype.check = function (caches) {
        var cacheList = !caches
            ? []
            : caches.constructor === Array
                ? caches
                : [caches];
        if (process.env.NODE_ENV !== 'production') {
            for (var _i = 0, cacheList_1 = cacheList; _i < cacheList_1.length; _i++) {
                var cache = cacheList_1[_i];
                if (this.caches.indexOf(cache) < 0) {
                    throw new Error('Router must return a subset from submitted caches');
                }
            }
        }
        return cacheList;
    };
    return RoutingCache;
}(apollo_cache_1.ApolloCache));
exports.route = function (caches, router) {
    return new RoutingCache(caches, router);
};
var ApolloCacheRouter = (function () {
    function ApolloCacheRouter() {
    }
    ApolloCacheRouter.override = exports.override;
    ApolloCacheRouter.route = exports.route;
    return ApolloCacheRouter;
}());
exports.default = ApolloCacheRouter;

})));
//# sourceMappingURL=bundle.umd.js.map
